const assert = require('assert');
const { describe, it } = require('node:test');
const getMw = require('../lib/get-mw');

describe('getMw', () => {
   it('should correctly separate private and public mw files', () => {
      const files = [
         { path: '/path/to/mw', name: 'public.mw.js' },
         { path: '/path/to/mw', name: '#private.mw.js' },
         { path: '/path/to/other', name: 'notmw.txt' }
      ];
      const { mw, privateMw } = getMw(files);

      assert.strictEqual(Object.keys(mw).length, 1);
      assert.strictEqual(Object.keys(privateMw).length, 1);
      assert.strictEqual(mw['/path/to/mw'].split(/\\/).join('/'), '/path/to/mw/public.mw.js');
      assert.strictEqual(privateMw['/path/to/mw'].split(/\\/).join('/'), '/path/to/mw/#private.mw.js');
   });

   it('should return empty objects for empty input', () => {
      const files = [];
      const { mw, privateMw } = getMw(files);

      assert.deepStrictEqual(mw, {});
      assert.deepStrictEqual(privateMw, {});
   });

   it('should ignore files without .js extension', () => {
      const files = [
         { path: '/path/to/mw', name: 'public.mw.txt' },
         { path: '/path/to/mw', name: '#private.mw.md' }
      ];
      const { mw, privateMw } = getMw(files);

      assert.deepStrictEqual(mw, {});
      assert.deepStrictEqual(privateMw, {});
   });

   it('should ignore non-middleware files', () => {
      const files = [
         { path: '/path/to/mw', name: 'example.txt' },
         { path: '/path/to/mw', name: 'script.js' }
      ];
      const { mw, privateMw } = getMw(files);

      assert.deepStrictEqual(mw, {});
      assert.deepStrictEqual(privateMw, {});
   });

   it('should correctly handle a mix of file types', () => {
      const files = [
         { path: '/path/to/mw', name: 'public.mw.js' },
         { path: '/path/to/mw', name: 'example.txt' },
         { path: '/path/to/mw', name: '#private.mw.js' },
         { path: '/path/to/mw', name: 'script.js' }
      ];
      const { mw, privateMw } = getMw(files);

      assert.strictEqual(Object.keys(mw).length, 1);
      assert.strictEqual(Object.keys(privateMw).length, 1);
   });

   it('should handle files with special characters in names', () => {
      const files = [
         { path: '/path/to/mw', name: 'public_special@.mw.js' },
         { path: '/path/to/mw', name: '#private&special.mw.js' }
      ];
      const { mw, privateMw } = getMw(files);
      assert.strictEqual(Object.keys(mw).length, 1);
      assert.strictEqual(Object.keys(privateMw).length, 1);
   });

});
