const assert = require('assert');
const { describe, it } = require('node:test');
const getRoutes = require('../lib/get-routes');

describe('getRoutes', () => {

    it('should return an empty array for empty input', () => {
        const files = [];
        const mwObj = { mw: {}, privateMw: {} };
        const dirPath = '/path/to/dir';

        const routes = getRoutes(files, mwObj, dirPath);

        assert.deepStrictEqual(routes, []);
    });

    it('should ignore files with invalid HTTP methods', () => {
        const files = [
            { path: '/path/to/dir', name: 'home.invalidmethod.js' },
            { path: '/path/to/dir', name: 'about.notamethod.js' }
        ];
        const mwObj = { mw: {}, privateMw: {} };
        const dirPath = '/path/to/dir';

        const routes = getRoutes(files, mwObj, dirPath);

        assert.deepStrictEqual(routes, []);
    });

    it('should ignore non-JavaScript files', () => {
        const files = [
            { path: '/path/to/dir', name: 'home.get.txt' },
            { path: '/path/to/dir', name: 'about.post.md' }
        ];
        const mwObj = { mw: {}, privateMw: {} };
        const dirPath = '/path/to/dir';

        const routes = getRoutes(files, mwObj, dirPath);

        assert.deepStrictEqual(routes, []);
    });

    it('should integrate middleware correctly', () => {
        const files = [
            { path: '/path/to/dir', name: 'home.get.js' }
        ];
        const mwObj = {
            mw: { '/path/to/dir': '/path/to/dir/common.mw.js' },
            privateMw: { '/path/to/dir': '/path/to/dir/#private.mw.js' }
        };
        const dirPath = '/path/to/dir';

        const routes = getRoutes(files, mwObj, dirPath);

        assert.strictEqual(routes.length, 1);
        assert(routes[0].pathes.includes(('/path/to/dir/home.get.js').split('/').join('\\')));
        assert(routes[0].pathes.includes('/path/to/dir/common.mw.js'));
        assert(routes[0].pathes.includes('/path/to/dir/#private.mw.js'));
    });

    it('should process valid route files correctly', () => {
        const files = [
            { path: '/path/to/dir', name: 'home.get.js' },
            { path: '/path/to/dir', name: 'about.post.js' }
        ];
        const mwObj = { mw: {}, privateMw: {} };
        const dirPath = '/path/to/dir';

        const routes = getRoutes(files, mwObj, dirPath);

        assert.strictEqual(routes.length, 2);
        assert.strictEqual(routes[0].method, 'get');
        assert.strictEqual(routes[1].method, 'post');
    });

    it('should correctly handle a mix of valid and invalid files', () => {
        const files = [
            { path: '/path/to/dir', name: 'home.get.js' },
            { path: '/path/to/dir', name: 'invalid.file.txt' },
            { path: '/path/to/dir', name: 'about.post.js' },
            { path: '/path/to/dir', name: 'wrongmethod.update.js' }
        ];
        const mwObj = { mw: {}, privateMw: {} };
        const dirPath = '/path/to/dir';

        const routes = getRoutes(files, mwObj, dirPath);

        assert.strictEqual(routes.length, 2);
    });

    it('should correctly handle files in nested directories', () => {
        const files = [
            { path: '/path/to/dir/subdir', name: 'contact.get.js' }
        ];
        const mwObj = { mw: {}, privateMw: {} };
        const dirPath = '/path/to/dir';

        const routes = getRoutes(files, mwObj, dirPath);

        assert.strictEqual(routes.length, 1);
    });

    it('should correctly handle mismatched middleware files', () => {
        const files = [
            { path: '/path/to/dir', name: 'home.get.js' }
        ];
        const mwObj = {
            mw: { '/wrong/path': '/wrong/path/common-mw.js' },
            privateMw: { '/wrong/path': '/wrong/path/#private-mw.js' }
        };
        const dirPath = '/path/to/dir';

        const routes = getRoutes(files, mwObj, dirPath);

        assert.strictEqual(routes.length, 1);
        assert.strictEqual(routes[0].pathes.length, 1); // Only the route file, no middleware files
    });

});
