const assert = require('assert');
const { describe, it, after } = require('node:test');
const routes = require('../lib/routes');
const { outputFileSync, removeSync } = require('fs-extra');
const { join, parse } = require('path')

const testDir = join(__dirname, 'testRoutes');

const routeFiles = [
   "/.get.js",
   "/login.get.js",
   "/login.post.js",
   "/logout.get.js",
   "/register.get.js",
   "/register.post.js",
   "/not-a-route.js", // not valid route

   "/dashboard/.get.js",
   "/dashboard/auth.mw.js",
   "/dashboard/#.mw.js", // privat mw
   "/dashboard/profile.get.js",
   "/dashboard/change_password.post.js",
   "/dashboard/not-valid.get.js",

   "/dashboard/files/files_store.mw.js",
   "/dashboard/files/files_check.mw.js",
   "/dashboard/files/file.get.js",
   "/dashboard/files/file.post.js",
   "/dashboard/files/file.delete.js"
];

routeFiles.forEach(path => {
   let { name } = parse(path)
   name = name.split('.')[0]
   name = name === '#' ? 'mw' : name
   const content = path.endsWith('not-valid.get.js')
      ? 'dgdfg'
      : `module.exports = function ${name}() {}`
   outputFileSync(join(testDir, path), content, 'utf-8')
});

const { readyRoutes, errors } = routes(testDir, '/')

describe('routes', () => {
   after(() => removeSync(testDir))

   it('should correctly add routes', () => {
      assert.strictEqual(readyRoutes.length, 12);

      // Проверка наличия конкретных роутов и их middleware
      assert(readyRoutes.some(route => route.path === '/' && route.method === 'get'));
      assert(readyRoutes.some(route => route.path === '/login' && route.method === 'get'));

      // Проверка корректности middleware
      const dashboardRoute = readyRoutes.find(route => route.path === '/dashboard' && route.method === 'get');
      assert(dashboardRoute && dashboardRoute.fns.length === 3); // Убедитесь, что есть 3 MW

      assert.strictEqual(errors.length, 1); // Ожидается одна ошибка

   });

   it('should correctly add routes and middleware in the right order', () => {

      // Проверка порядка и присутствия middleware для /dashboard
      const dashboardRoute = readyRoutes.find(route => route.path === '/dashboard' && route.method === 'get');
      if (dashboardRoute && dashboardRoute.fns.length > 0) {
         assert.strictEqual(dashboardRoute.fns[0].name, 'auth'); // Общий MW
         assert.strictEqual(dashboardRoute.fns[1].name, 'mw'); // Приватный MW
         assert.strictEqual(dashboardRoute.fns[2].name, ''); // Анонимная функция роута
      }

      // Проверка порядка и присутствия middleware для /dashboard/profile
      const profileRoute = readyRoutes.find(route => route.path === '/dashboard/profile' && route.method === 'get');
      if (profileRoute && profileRoute.fns.length > 0) {
         assert.strictEqual(profileRoute.fns[0].name, 'auth'); // Общий MW
         assert.strictEqual(profileRoute.fns[1].name, 'mw'); // Приватный MW
         assert.strictEqual(profileRoute.fns[2].name, 'profile'); // Функция роута
      }
   });

   it('should contain all expected routes', () => {
      const { readyRoutes } = routes(testDir, '/');

      const expectedRoutes = [
         '/', '/login', '/logout', '/register',
         '/dashboard', '/dashboard/profile',
         '/dashboard/files/file'
      ];

      expectedRoutes.forEach(expectedPath => {
         assert(readyRoutes.some(route => route.path === expectedPath));
      });
   });

   it('urlPath', () => {
      const { readyRoutes, errors } = routes(testDir, '/some')
      const index = Math.floor(Math.random() * (readyRoutes.length))
      assert(readyRoutes[index].path.startsWith('/some/'))
   })

});
